
/* --- Administrator-configurable defaults. ------------------------------ */

#define DEFAULT_TITLE_FONT \
	"-*-lucida-bold-r-normal-sans-14-*-*-*-p-*-iso8859-1"
#define DEFAULT_POPUP_FONT \
	"-*-lucida-medium-r-normal-sans-12-*-*-*-p-*-iso8859-1"
#define DEFAULT_TERMINAL \
	"xterm"
#define DEFAULT_BORDER \
	6
#define MAXHIDDEN \
	32

/* --- End of administrator-configurable defaults. ----------------------- */


#define EXIT_FAILURE 1
#define EXIT_SUCCESS 0




#define USED(x) (x=x);

/*
 * Window manager mode. wm is in one of three modes: it's either getting
 * user input to move/reshape a window, getting user input to make a
 * selection from the menu, or it's `idle' --- responding to events arriving
 * from the server, but not directly interacting with the user.
 * OK, so I lied: there's a fourth mode so that we can tell when wm's still
 * initialising.
 */
typedef enum {
	wm_initialising, wm_idle, wm_reshaping, wm_menu_up
} Mode;

/*
 *	Window internal state. Yuck.
 */
typedef enum {
	IPendingReparenting, INormal, IPendingDeletion
} IState ;

/*
 *	Window edge, used in resizing. The `edge' ENone is used to signify a
 * window move rather than a resize. The code is sufficiently similar that
 * this isn't a special case to be treated separately.
 *
 * These MUST be in the same order as they appear in cursor.c !
 */
typedef enum {
	ETopLeft,		ETop,		ETopRight,
	ERight,			ENone,	ELeft,
	EBottomLeft,	EBottom,	EBottomRight
} Edge ;

typedef struct Client Client;
struct Client {
	Window window;			/* Client's window. */
	Window parent;			/* Window manager frame. */
	Window trans;			/* Window that client is a transient for. */

	Client *next;			/* Next window in client list. */

	int border;				/* Client's original border width. */

	XSizeHints size;		/* Client's current geometry information. */
	int state;				/* Window state, as per ICCCM. See <X11/Xutil.h> */

	IState internal_state;
	int proto;

	int accepts_focus;	/* Can this window make use of keyboard events? */

	char *name;				/* Name used for title in frame and menu. */

	/* Colourmap scum. */
	Colormap cmap;
	int ncmapwins;
	Window *cmapwins;
	Colormap *wmcmaps;
};


/*
 *	c->proto is a bitarray of these
 */
enum {
	Pdelete = 1,
	Ptakefocus = 2
};

/*
 * This should really have been in X.h --- if you select both ButtonPress
 * and ButtonRelease events, the server makes an automatic grab on the
 * pressed button for you. This is almost always exactly what you want.
 */
#define ButtonMask (ButtonPressMask | ButtonReleaseMask)

/*	wm.c	*/
extern Mode mode;
extern int start_x;
extern int start_y;
extern Display	*dpy;
extern int display_width;
extern int display_height;
extern Window	root;
extern Window	popup;
extern GC gc;
extern GC gc_thin;
extern GC popup_gc;
extern XFontStruct *font;
extern XFontStruct *popup_font;
extern Atom wm_state;
extern Atom wm_change_state;
extern Atom wm_protocols;
extern Atom wm_delete;
extern Atom wm_take_focus;
extern Atom wm_colormaps;
extern unsigned long black;
extern unsigned long white;
extern Bool shape;
extern int shape_event;
extern void	shell(void);
extern void	SendConfigureNotify(Client *);
extern int	titleheight(void);

/*	client.c	*/
extern Edge interacting_edge;
extern Client *Client_Get(Window);
extern Client *Client_Add(Window);
extern void Client_MakeSane(Client *, Edge, int *, int *, int *, int *);
extern void Client_DrawBorder(Client *, int);
extern void setactive(Client *, int);
extern void	Client_ReshapeEdge(Client *, Edge);
extern void	Client_Move(Client*);
extern void Client_SetState(Client *, int);
extern void Client_Close(Client *);
extern void Client_Remove(Client *);
extern void Client_FreeAll(void);
extern void Client_ColourMap(XEvent*);
extern int hidden(Client *);
extern int withdrawn(Client *);
extern int normal(Client *);
extern Client *current;

/*	cursor.c	*/
extern Cursor root_cursor;
extern Cursor frame_cursor;
extern Cursor edgecursor(Edge edge);
extern void Cursor_Initialise(void);

/*	disp.c	*/
extern void dispatch(XEvent *);

/*	error.c	*/
extern int	ignore_badwindow;
extern int	ErrorHandler(Display *, XErrorEvent *);
extern void	Panic(char*);

/*	manage.c	*/
extern void	getWindowName(Client *);
extern void	manage(Client *, int);
extern void	withdraw(Client *);
extern void	cmapfocus(Client *);
extern void	getcmaps(Client *);
extern void	gettrans(Client *);
extern void	Terminate(int);

/*	mouse.c	*/
extern int current_item;
extern int current_items;
extern void getmouse(int * /*pointer_x*/, int * /*pointer_y*/);
extern void hide(Client *);
extern void unhidec(Client *, int);
extern void update_menu(Client *);
extern int menu_whichitem(int, int);
extern void menu_dimensions(int *, int *);
extern void menuhit(XButtonEvent *);
extern void unhide(int, int);
extern void menu_expose(void);
extern void menu_motionnotify(XEvent *);

/*	shape.c	*/
extern int	shapeevent(XEvent *);
extern int	servershapes(void);
extern int	isshaped(Window);

/*	resource.c	*/
extern char	*font_name;
extern char	*popup_font_name;
extern char	*terminal;
extern int	border;
extern void	get_resources(void);
